/* File:      spatmod.C
   Date:      6/21/02  (written fall 2001)
   Author:    C. Paciorek
   Purpose:   contains the main function for the Bayesian GP spatial model for my dissertation
   Compile:   g++ -lg2c -I/usr/statlocal/vni/CTT3.0/include -L/usr/statlocal/vni/CTT3.0/lib/lib.linux -limslcmath -limslcstat -lm -Xlinker -rpath -Xlinker /usr/statlocal/vni/CTT3.0/lib/lib.linux
              -L$(HOME)/lib/Proj/lib -lproj
   Usage:     nice +19 mod ~/research/dat/eady1.dat ~/research/dat/lat1.dat ~/research/dat/lon1.dat ~/research/dat/kernel_lat.dat ~/research/dat/kernel_lon.dat ~/research/dat/perm.dat ~/research/dat/invp.dat ~/research/dat/years.dat ~/research/parmfiles/spatinit1-1.dat ~/research/parmfiles/spatfinal1-1a.dat ~/research/parmfiles/spatparm1-1a.dat /home/HYDRA4/paciorek/output/spatmod/spatout1-1a.dat ~/research/parmfiles/idumA2S-100 3.14 1 7 19 9 10000 10 1000 5
   Arguments: data_file latitudes_file longitudes_file kernel_latitudes_file kernel_longitudes_file permutation_file inverse_permutation_file years_file initialvals_file finalvals_file  parm_file output_file max_nonzero_correlation statprior_indicator num_lats num_lons num_basis_kernels number_of_iterations storage_interval stored_its_per_outputfile fastparm_sample_interval
   Requires:  base.h, ran1.h, locations.h, chain.h
   Reference: my proposal!, Makefile for other code files
*/

/* Code files:
   spatmod.C     main
   projection.C  code for geographic and projection stuff
   generic_functions.C   basic functions and correlation functions
   dataset.C     data storage and likelihood calc
   parm.C        parameter class: initialization,storage,updating
   cov_mat.C     covariance matrix code
   permtools.C   basic permutation functions
   chain.C       stores data and parameters and does MCMC steps
   gasdev.C      random normal generation
   genqmd.f      permutation function for achieving maximal Cholesky sparseness
   gsfct.f       numeric Cholesky factorization
   smbfct.f      symbolic Cholesky factorization -- sets up sparse structure
   locations.C   loads location info, computes adjacency, permutes by calling genqmd
   ran1.C        random uniform generation
   transform.C   transformation functions
   density.C     density functions
   mat_pp.h      storage via array of pointers to pointers (arrays)
*/

#include "base.h"
#include "ran1.h"
#include "locations.h"
#include "chain.h"
#include "corr.h"  /* for initizliation call to matern */

int* idum;

int printout;

locn_info* locations;
dataset* data;

int prior;    // prior for data covariance is STATIONARY or NONSTATIONARY

int main(int argc, char **argv)
{
  printout=0;
    // argv[1] is data file name; read in BY_COL from ragged format
    // argv[2] is lat file
    // argv[3] is lon file
    // argv[4] is the lat file for the basis kernels
    // argv[5] is the lon file for the basis kernels
    // argv[6] is the permutation file for permuting locations
    // argv[7] is the inverse permutation file for permuting locations
    // argv[8] is years file (included years are modelled; excluded used in cross-validation
    // argv[9] is initialization value file
    // argv[10] is final value output file
    // argv[11] is fixed parameter value file
    // argv[12] is output file 
    // argv[13] is range
    // argv[14] is 0 or "stat" for stationary data cov and 1 or "nonstat" for nonstationary data cov and the filename if data cov is fixed
    // argv[15] is the number of latitudes in grid
    // argv[16] is the number of longitudes in grid
    // argv[17] is the number of basis kernels
    // argv[18] is number of iterations
    // argv[19] is the interval between iterate storage
    // argv[20] is the number of stored iterates in each output file
    // argv[21] is the interval at which the mu, delta and data_kappa parameters, which mix quickly, are sampled
    
    // lat data file should read 70 67.5 65 ...  20
    // lon data file: 0 2.5 5 .... 357.5 

  int i;


  cout << "Running spatial model with: " << endl;
  cout << "Data file: " << argv[1] << endl;
  cout << "Lat file: " << argv[2] << endl;
  cout << "Lon file: " << argv[3] << endl;
  cout << "Basis kernel lat file: " << argv[4] << endl;
  cout << "Basis kernel lon file: " << argv[5] << endl;
  cout << "Permutation file: " << argv[6] << endl;
  cout << "Inverse permutation file: " << argv[7] << endl;
  cout << "Years file: " << argv[8] << endl;
  cout << "Initial value file: " << argv[9] << endl;
  cout << "Final value file: " << argv[10] << endl;
  cout << "Parameter file: " << argv[11] << endl;
  cout << "Output file: " << argv[12] << endl;
  cout << "Range: " << argv[13] << endl;
  cout << "Type of covariance: " << argv[14] << endl;
  cout << "Number of lats: " << argv[15] << endl;
  cout << "Number of lons: " << argv[16] << endl;
  cout << "Number of basis kernels: " << argv[17] << endl;
  cout << "Number of iterations: " << argv[18] << endl;
  cout << "Iterate storage interval: " << argv[19] << endl;
  cout << "Write-to-file interval: " << argv[20] << endl;
  cout << "Fast-mixing parameter sampling interval: " << argv[21] << endl;


  int num_kernels=0;

  string datacorr_structure=argv[14];
  if(datacorr_structure=="0" || datacorr_structure=="stat" ||  datacorr_structure=="stationary"){
      prior=STATIONARY;
  }
  else{
    if(datacorr_structure=="2" || datacorr_structure=="nonstat" ||  datacorr_structure=="nonstationary"){
        prior=NON_STATIONARY;
        num_kernels=atoi(argv[17]);
    }
    else{
      prior=FIXED;
    }
  }
  
  int its=atoi(argv[18]);
  int storage_interval=atoi(argv[19]);
  int storage_space=atoi(argv[20]);
  int fastparm_interval=atoi(argv[21]);

  double range=atof(argv[13]);

  locations=new locn_info(argv[2],argv[3],argv[4],argv[5],argv[6],argv[7],GRID,range,atoi(argv[15]),atoi(argv[16]),num_kernels);    // deal with lat/lon coordinates, adjacency info, permutation

  chain* this_chain=new chain(locations->num_gridpoints,argv[1],argv[8],argv[9],argv[10],argv[11],argv[12],its,storage_interval,storage_space,fastparm_interval,datacorr_structure);                  // load initial values and set up parameters


  time_t t1,t2,diff;
  char *timeout;

  t1=time(0);
  timeout=ctime(&t1);
  cout << "Start at: " << timeout << endl; cout.flush();

  for(i=1;i<=its;i++){
    this_chain->do_a_step(i); 
    this_chain->store(i);       // store iterates      
  }

  t2=time(0);
  timeout= ctime(&t2);
  diff=t2-t1;
  cout << "Time for " << its << " MCMC iterations: " << diff << ".\n Ended at: " << timeout << endl;cout.flush();

  this_chain->report(its,FINAL);         
  delete this_chain;
  delete locations;
  if(its>9000){                       // send mail when done
    string cmd="source mail.csh ";
    cmd=cmd+argv[12]+" "+argv[18];
    system(cmd.c_str());
  }
}


